@@ -1,11 +1,15 @@
 mod bars;
 mod cli;
+mod config;
 mod format;
 mod theme;
 
 use cli::{MacchinaColor, Opt};
+use colored::Colorize;
 use std::io;
 use structopt::StructOpt;
+use theme::Themes;
+
 #[macro_use]
 extern crate lazy_static;
 
@@ -63,21 +67,14 @@ fn draw_ascii(ascii: Text<'static>, tmp_buffer: &mut Buffer) -> Rect {
     ascii_rect
 }
 
-fn draw_readout_data(
-    data: Vec<Readout>,
-    theme: Box<dyn Theme>,
-    buf: &mut Buffer,
-    area: Rect,
-    show_box: bool,
-    palette: bool,
-) {
-    let mut list = ReadoutList::new(data, &theme).palette(palette);
+fn draw_readout_data(data: Vec<Readout>, theme: Theme, buf: &mut Buffer, area: Rect, config: &Opt) {
+    let mut list = ReadoutList::new(data, &theme).palette(config.palette);
 
-    if show_box {
+    if !config.no_box {
         list = list
             .block_inner_margin(Margin {
-                horizontal: 1,
-                vertical: 1,
+                horizontal: config.box_inner_margin_x,
+                vertical: config.box_inner_margin_y,
             })
             .block(
                 Block::default()
@@ -90,8 +87,23 @@ fn draw_readout_data(
     list.render(area, buf);
 }
 
-fn create_theme(opt: &Opt) -> Box<dyn Theme> {
-    let mut theme = opt.theme.create_instance();
+fn create_theme(opt: &Opt) -> Theme {
+    let mut theme;
+    if let Some(opt_theme) = &opt.theme {
+        if let Ok(ts) = theme::Themes::from_str(opt_theme) {
+            theme = Theme::new(ts);
+        } else if let Ok(custom_theme) = theme::CustomTheme::get_theme(opt_theme) {
+            theme = Theme::from(custom_theme);
+        } else {
+            println!(
+                "\x1b[33mWarning:\x1b[0m Invalid theme {}, falling back to default",
+                opt_theme
+            );
+            theme = Theme::default();
+        }
+    } else {
+        theme = Theme::default();
+    }
     let color_variants = MacchinaColor::variants();
     let make_random_color = || {
         let mut random = rand::thread_rng();
@@ -175,8 +187,68 @@ fn select_ascii(small: bool) -> Option<Text<'static>> {
     }
 }
 
+fn list_themes() {
+    let themes = Themes::variants();
+    themes
+        .iter()
+        .for_each(|x| println!("• {} (Built-in)", x.bright_green()));
+
+    if let Some(dir) = dirs::data_local_dir() {
+        let entries = libmacchina::extra::list_dir_entries(&dir.join("macchina/themes"));
+        if !entries.is_empty() {
+            let custom_themes = entries.iter().filter(|&x| {
+                if let Some(ext) = libmacchina::extra::path_extension(&x) {
+                    ext == "json"
+                } else {
+                    false
+                }
+            });
+
+            if custom_themes.clone().count() == 0 {
+                println!(
+                    "\nNo custom themes were found in {}",
+                    dir.join("macchina/themes")
+                        .to_string_lossy()
+                        .bright_yellow()
+                )
+            }
+
+            custom_themes.for_each(|x| {
+                if let Some(theme) = x.file_name() {
+                    let name = theme.to_string_lossy().replace(".json", "");
+                    println!("• {}", name.bright_blue());
+                }
+            });
+        }
+    }
+}
+
 fn main() -> Result<(), io::Error> {
-    let opt = Opt::from_args();
+    let mut opt: Opt;
+    let config_opt = Opt::from_config();
+    let arg_opt = Opt::from_args();
+
+    if arg_opt.export_config {
+        println!("{}", toml::to_string(&arg_opt).unwrap());
+        return Ok(());
+    }
+
+    if let Ok(mut config_opt) = config_opt {
+        config_opt.patch_args(Opt::from_args());
+        opt = config_opt;
+        let conflicts = opt.check_conflicts();
+        if !conflicts.is_empty() {
+            println!("\x1b[33mWarning:\x1b[0m Conflicting keys in config file:");
+            for conflict in conflicts {
+                println!("• {}", conflict);
+            }
+            opt = arg_opt;
+        }
+    } else {
+        println!("\x1b[33mWarning:\x1b[0m Invalid config file");
+        opt = arg_opt;
+    }
+
     let should_display = should_display(&opt);
     let theme = create_theme(&opt);
     let readout_data = data::get_all_readouts(&opt, &theme, should_display);
@@ -186,16 +258,21 @@ fn main() -> Result<(), io::Error> {
         return Ok(());
     }
 
+    if opt.list_themes {
+        list_themes();
+        return Ok(());
+    }
+
     let mut backend = create_backend();
     let mut tmp_buffer = Buffer::empty(Rect::new(0, 0, 500, 50));
 
     let ascii_area;
 
-    if let Some(file_path) = opt.custom_ascii {
+    if let Some(ref file_path) = opt.custom_ascii {
         let file_path = PathBuf::from(file_path);
         let ascii_art;
         match opt.custom_ascii_color {
-            Some(color) => {
+            Some(ref color) => {
                 ascii_art = ascii::get_ascii_from_file_override_color(
                     &file_path,
                     color.get_color().to_owned(),
@@ -208,13 +285,13 @@ fn main() -> Result<(), io::Error> {
         };
 
         // If the file is empty just default to disabled
-        if ascii_art.width() != 0 && ascii_art.height() < 50 {
+        if ascii_art.width() != 0 && ascii_art.height() < 50 && !opt.small_ascii {
             // because tmp_buffer height is 50
             ascii_area = draw_ascii(ascii_art.to_owned(), &mut tmp_buffer);
         } else {
             ascii_area = Rect::new(0, 1, 0, tmp_buffer.area.height - 1);
         }
-    } else if readout_data.len() <= 6 {
+    } else if readout_data.len() <= 6 || opt.small_ascii {
         ascii_area = match (opt.no_ascii, select_ascii(true)) {
             (false, Some(ascii)) => draw_ascii(ascii.to_owned(), &mut tmp_buffer),
             _ => Rect::new(0, 1, 0, tmp_buffer.area.height - 1),
@@ -238,8 +315,7 @@ fn main() -> Result<(), io::Error> {
             tmp_buffer_area.width - ascii_area.width - 4,
             ascii_area.height,
         ),
-        !opt.no_box,
-        opt.palette,
+        &opt,
     );
 
     write_buffer_to_console(&mut backend, &mut tmp_buffer)?;
